#ifndef OSP_CALIB_CAMERA_H
#define OSP_CALIB_CAMERA_H

#include <osp/calib/intrinsic.h>
#include <osp/calib/extrinsic.h>
#include <osp/exports/calib_export.h>
#include <osp/types/mat.h>
#include <osp/types/ndarray.h>

namespace osp {

class OSP_CALIB_EXPORT Camera
{
public:
    Camera() = default;
    virtual ~Camera() = default;
    Camera(const Camera &other);
    Camera(Camera && other);
    Camera & operator = (Camera other);
    friend void swap(Camera &a, Camera &b);

    static Camera readFromFile(const std::string &filename);
    void saveToFile(const std::string &filename) const;

    const Intrinsic &intrinsic() const;
    Intrinsic &intrinsic();
    const Extrinsic &extrinsic() const;
    Extrinsic &extrinsic();

    void setIntrinsic(Intrinsic intrinsic);
    void setExtrinsic(Extrinsic extrinsic);

private:
    Intrinsic m_intrinsic;
    Extrinsic m_extrinsic;
};

OSP_CALIB_EXPORT bool operator==(const Camera &lhs, const Camera &rhs);
OSP_CALIB_EXPORT bool operator!=(const Camera &lhs, const Camera &rhs);

template<typename T>
typename Reprojected<T>::type imgToWrld(const T &imgPt, const Camera &cam)
{
    return camToWrld(imgToCam(imgPt, cam.intrinsic()), cam.extrinsic());
}

template<typename T>
typename Projected<T>::type wrldToImg(const T &wrldPt, const Camera &cam)
{
    return camToImg(wrldToCam(wrldPt, cam.extrinsic()), cam.intrinsic());
}

template<typename T>
Matrix<typename Reprojected<T>::type> imgToWrld(const Matrix<T> &imgPt, const Camera &cam)
{
    return camToWrld(imgToCam(imgPt, cam.intrinsic()), cam.extrinsic());
}

template<typename T>
Matrix<typename Projected<T>::type> wrldToImg(const Matrix<T> &wrldPt, const Camera &cam)
{
    return camToImg(wrldToCam(wrldPt, cam.extrinsic()), cam.intrinsic());
}

OSP_CALIB_EXPORT bool readFromNode(Camera &camera, const boost::property_tree::ptree &node);
OSP_CALIB_EXPORT void writeToNode(const Camera &camera, boost::property_tree::ptree &node);

}

#endif /* OSP_CALIB_CAMERA_H */
