#ifndef OSP_CALIB_CAMERAMODELCONVERTER_H
#define OSP_CALIB_CAMERAMODELCONVERTER_H

#include <osp/calib/camera.h>

namespace osp {

class OSP_CALIB_EXPORT CameraModelConverter
{
public:
    CameraModelConverter() = default;
    CameraModelConverter(Camera src, Camera dst);
    virtual ~CameraModelConverter() = default;

    Camera &source();
    const Camera &source() const;
    Camera &destination();
    const Camera &destination() const;

    void setSource(Camera src);
    void setDestination(Camera dst);

    Point2d<float> convert(const Point2d<float> &imgPtsDst, float reprojectionDistance = 1.0);
    Point2d<double> convert(const Point2d<double> &imgPtsDst, double reprojectionDistance = 1.0);

    template <typename T = float>
    Matrix<Point2d<T>> convert(T reprojectionDistance = 1.0);

    template <typename T>
    Matrix<Point2d<T>> convert(Matrix<Point2d<T>> imgPtsDst, T reprojectionDistance = 1.0);

private:
    Camera m_src;
    Camera m_dst;
};

template <typename T>
Matrix<Point2d<T>> CameraModelConverter::convert(T reprojectionDistance)
{
    Matrix<Point2d<T>> imgPtsDst = m_dst.intrinsic().imagePoints<T>();
    return convert<T>(imgPtsDst, reprojectionDistance);
}

template <typename T>
Matrix<Point2d<T>> CameraModelConverter::convert(Matrix<Point2d<T>> imgPtsDst, T reprojectionDistance)
{
    auto rows = imgPtsDst.rows();
    auto cols = imgPtsDst.cols();

    auto imgPtsSrc = Matrix<Point2d<T>>{rows, cols};
    processElementwise(imgPtsDst, imgPtsSrc, [&](const auto &pt) { return this->convert(pt, reprojectionDistance); });
    return imgPtsSrc;
}



}

#endif /* OSP_CALIB_CAMERAMODELCONVERTER_H */
