#ifndef OSP_CALIB_PROJECTION_EPIPOLAREQUIDISTANT_H
#define OSP_CALIB_PROJECTION_EPIPOLAREQUIDISTANT_H

#include <boost/property_tree/ptree.hpp>
#include <osp/calib/projectionmodeltype.h>
#include <osp/exports/calib_export.h>
#include <osp/types/coordinates.h>
#include <osp/types/mat.h>

namespace osp {

class OSP_CALIB_EXPORT EpipolarEquidistantProjection
{
public:
    static ProjectionModelType type() { return ProjectionModelType::EpipolarEquidistant; }
    static std::string typeStr() { return "c_epipolar_equidistant"; }
};

OSP_CALIB_EXPORT bool operator==(const EpipolarEquidistantProjection &lhs, const EpipolarEquidistantProjection &rhs);
OSP_CALIB_EXPORT bool operator!=(const EpipolarEquidistantProjection &lhs, const EpipolarEquidistantProjection &rhs);

OSP_CALIB_EXPORT Polar<float> project(const EpipolarEquidistantProjection &projection, const Spherical<float> &camPt);
OSP_CALIB_EXPORT Spherical<float> reproject(const EpipolarEquidistantProjection &projection, const Polar<float> &normImgPt);
OSP_CALIB_EXPORT Point2d<float> project(const EpipolarEquidistantProjection &projection, const Point3d<float> &camPt);
OSP_CALIB_EXPORT Point3d<float> reproject(const EpipolarEquidistantProjection &projection, const Point2d<float> &normImgPt);
OSP_CALIB_EXPORT Polar<double> project(const EpipolarEquidistantProjection &projection, const Spherical<double> &camPt);
OSP_CALIB_EXPORT Spherical<double> reproject(const EpipolarEquidistantProjection &projection, const Polar<double> &normImgPt);
OSP_CALIB_EXPORT Point2d<double> project(const EpipolarEquidistantProjection &projection, const Point3d<double> &camPt);
OSP_CALIB_EXPORT Point3d<double> reproject(const EpipolarEquidistantProjection &projection, const Point2d<double> &normImgPt);

OSP_CALIB_EXPORT bool readFromNode(EpipolarEquidistantProjection &projection, const boost::property_tree::ptree &node);
OSP_CALIB_EXPORT void writeToNode(const EpipolarEquidistantProjection &projection, boost::property_tree::ptree &node);

}


#endif /* OSP_CALIB_PROJECTION_EPIPOLAREQUIDISTANT_H */
