#ifndef OSP_CALIB_PROJECTION_EQUIANGULAR_H
#define OSP_CALIB_PROJECTION_EQUIANGULAR_H

#include <boost/property_tree/ptree.hpp>
#include <osp/calib/projectionmodeltype.h>
#include <osp/exports/calib_export.h>
#include <osp/types/coordinates.h>
#include <osp/types/mat.h>

namespace osp {

class OSP_CALIB_EXPORT EquiangularProjection
{
public:
    EquiangularProjection();

    static ProjectionModelType type() { return ProjectionModelType::Equiangular; }
    static std::string typeStr() { return "c_equiangular_r_sym"; }

    void setK(double k);
    double k() const;

private:
    double m_k;
};

OSP_CALIB_EXPORT bool operator==(const EquiangularProjection &lhs, const EquiangularProjection &rhs);
OSP_CALIB_EXPORT bool operator!=(const EquiangularProjection &lhs, const EquiangularProjection &rhs);

OSP_CALIB_EXPORT Polar<float> project(const EquiangularProjection &projection, const Spherical<float> &camPt);
OSP_CALIB_EXPORT Spherical<float> reproject(const EquiangularProjection &projection, const Polar<float> &normImgPt);
OSP_CALIB_EXPORT Point2d<float> project(const EquiangularProjection &projection, const Point3d<float> &camPt);
OSP_CALIB_EXPORT Point3d<float> reproject(const EquiangularProjection &projection, const Point2d<float> &normImgPt);
OSP_CALIB_EXPORT Polar<double> project(const EquiangularProjection &projection, const Spherical<double> &camPt);
OSP_CALIB_EXPORT Spherical<double> reproject(const EquiangularProjection &projection, const Polar<double> &normImgPt);
OSP_CALIB_EXPORT Point2d<double> project(const EquiangularProjection &projection, const Point3d<double> &camPt);
OSP_CALIB_EXPORT Point3d<double> reproject(const EquiangularProjection &projection, const Point2d<double> &normImgPt);


OSP_CALIB_EXPORT bool readFromNode(EquiangularProjection &projection, const boost::property_tree::ptree &node);
OSP_CALIB_EXPORT void writeToNode(const EquiangularProjection &projection, boost::property_tree::ptree &node);

}


#endif /* OSP_CALIB_PROJECTION_EQUIANGULAR_H */
