#ifndef OSP_CALIB_PROJECTION_GENERIC_ASYMMETRIC_H
#define OSP_CALIB_PROJECTION_GENERIC_ASYMMETRIC_H

#include <osp/calib/projection/genericradsym.h>

namespace osp {

class OSP_CALIB_EXPORT GenericAsymmetricProjection
{
public:
    GenericAsymmetricProjection();

    static ProjectionModelType type() { return ProjectionModelType::GenericAsymmetric; }
    static std::string typeStr() { return "c_generic_a_sym"; }

    size_t numParamsI() const;
    size_t numParamsJ() const;
    size_t numParamsL() const;
    size_t numParamsM() const;

    const GenericRadialSymmetricProjection &radSymProjection() const;
    double i(size_t idx) const;
    double j(size_t idx) const;
    double l(size_t idx) const;
    double m(size_t idx) const;

    void setRadSymProjection(const GenericRadialSymmetricProjection &radSymProjection);
    void setI(size_t idx, double i);
    void setJ(size_t idx, double j);
    void setL(size_t idx, double l);
    void setM(size_t idx, double m);

private:
    GenericRadialSymmetricProjection m_radsym;
    std::vector<double> m_i;
    std::vector<double> m_j;
    std::vector<double> m_l;
    std::vector<double> m_m;
};

OSP_CALIB_EXPORT bool operator==(const GenericAsymmetricProjection &lhs, const GenericAsymmetricProjection &rhs);
OSP_CALIB_EXPORT bool operator!=(const GenericAsymmetricProjection &lhs, const GenericAsymmetricProjection &rhs);

OSP_CALIB_EXPORT Point2d<float> project(const GenericAsymmetricProjection &projection, const Point3d<float> &camPt);
OSP_CALIB_EXPORT Polar<float> project(const GenericAsymmetricProjection &projection, const Spherical<float> &camPt);
OSP_CALIB_EXPORT Point3d<float> reproject(const GenericAsymmetricProjection &projection, const Point2d<float> &camPt);
OSP_CALIB_EXPORT Spherical<float> reproject(const GenericAsymmetricProjection &projection, const Polar<float> &camPt);
OSP_CALIB_EXPORT Point2d<double> project(const GenericAsymmetricProjection &projection, const Point3d<double> &camPt);
OSP_CALIB_EXPORT Polar<double> project(const GenericAsymmetricProjection &projection, const Spherical<double> &camPt);
OSP_CALIB_EXPORT Point3d<double> reproject(const GenericAsymmetricProjection &projection, const Point2d<double> &camPt);
OSP_CALIB_EXPORT Spherical<double> reproject(const GenericAsymmetricProjection &projection, const Polar<double> &camPt);


OSP_CALIB_EXPORT bool readFromNode(GenericAsymmetricProjection &projection, const boost::property_tree::ptree &node);
OSP_CALIB_EXPORT void writeToNode(const GenericAsymmetricProjection &projection, boost::property_tree::ptree &node);

}

#endif /* OSP_CALIB_PROJECTION_GENERIC_ASYMMETRIC_H */
