#ifndef OSP_CALIB_PROJECTION_KANNALA9_H
#define OSP_CALIB_PROJECTION_KANNALA9_H

#include <vector>
#include <boost/property_tree/ptree.hpp>
#include <osp/calib/projectionmodeltype.h>
#include <osp/exports/calib_export.h>
#include <osp/types/coordinates.h>
#include <osp/types/mat.h>

namespace osp {

class OSP_CALIB_EXPORT GenericRadialSymmetricProjection
{
public:
    GenericRadialSymmetricProjection();

    static ProjectionModelType type() { return ProjectionModelType::GenericRadialSymmetric; }
    static std::string typeStr() { return "c_generic_r_sym"; }

    size_t numParamsK() const;
    void setNumParamsK(size_t count);
    double k(size_t i) const;
    void setK(size_t i, double val);

private:
    std::vector<double> m_k;
};

OSP_CALIB_EXPORT bool operator==(const GenericRadialSymmetricProjection &lhs, const GenericRadialSymmetricProjection &rhs);
OSP_CALIB_EXPORT bool operator!=(const GenericRadialSymmetricProjection &lhs, const GenericRadialSymmetricProjection &rhs);

OSP_CALIB_EXPORT Point2d<float> project(const GenericRadialSymmetricProjection &projection, const Point3d<float> &camPt);
OSP_CALIB_EXPORT Polar<float> project(const GenericRadialSymmetricProjection &projection, const Spherical<float> &camPt);
OSP_CALIB_EXPORT Point3d<float> reproject(const GenericRadialSymmetricProjection &projection, const Point2d<float> &camPt);
OSP_CALIB_EXPORT Spherical<float> reproject(const GenericRadialSymmetricProjection &projection, const Polar<float> &camPt);
OSP_CALIB_EXPORT Point2d<double> project(const GenericRadialSymmetricProjection &projection, const Point3d<double> &camPt);
OSP_CALIB_EXPORT Polar<double> project(const GenericRadialSymmetricProjection &projection, const Spherical<double> &camPt);
OSP_CALIB_EXPORT Point3d<double> reproject(const GenericRadialSymmetricProjection &projection, const Point2d<double> &camPt);
OSP_CALIB_EXPORT Spherical<double> reproject(const GenericRadialSymmetricProjection &projection, const Polar<double> &camPt);


OSP_CALIB_EXPORT bool readFromNode(GenericRadialSymmetricProjection &projection, const boost::property_tree::ptree &node);
OSP_CALIB_EXPORT void writeToNode(const GenericRadialSymmetricProjection &projection, boost::property_tree::ptree &node);

}


#endif /* OSP_CALIB_PROJECTION_KANNALA9_H */
