#ifndef OSP_CALIB_PROJECTION_PINHOLE_H
#define OSP_CALIB_PROJECTION_PINHOLE_H

#include <boost/property_tree/ptree.hpp>
#include <osp/calib/projectionmodeltype.h>
#include <osp/exports/calib_export.h>
#include <osp/types/coordinates.h>
#include <osp/types/mat.h>

namespace osp {

class OSP_CALIB_EXPORT PinholeProjection
{
public:
    static ProjectionModelType type() { return ProjectionModelType::Pinhole; }
    static std::string typeStr() { return "c_pinhole"; }
};

OSP_CALIB_EXPORT bool operator==(const PinholeProjection &lhs, const PinholeProjection &rhs);
OSP_CALIB_EXPORT bool operator!=(const PinholeProjection &lhs, const PinholeProjection &rhs);

OSP_CALIB_EXPORT Polar<float> project(const PinholeProjection &projection, const Spherical<float> &camPt);
OSP_CALIB_EXPORT Spherical<float> reproject(const PinholeProjection &projection, const Polar<float> &normImgPt);
OSP_CALIB_EXPORT Point2d<float> project(const PinholeProjection &projection, const Point3d<float> &camPt);
OSP_CALIB_EXPORT Point3d<float> reproject(const PinholeProjection &projection, const Point2d<float> &normImgPt);
OSP_CALIB_EXPORT Polar<double> project(const PinholeProjection &projection, const Spherical<double> &camPt);
OSP_CALIB_EXPORT Spherical<double> reproject(const PinholeProjection &projection, const Polar<double> &normImgPt);
OSP_CALIB_EXPORT Point2d<double> project(const PinholeProjection &projection, const Point3d<double> &camPt);
OSP_CALIB_EXPORT Point3d<double> reproject(const PinholeProjection &projection, const Point2d<double> &normImgPt);

OSP_CALIB_EXPORT bool readFromNode(PinholeProjection &projection, const boost::property_tree::ptree &node);
OSP_CALIB_EXPORT void writeToNode(const PinholeProjection &projection, boost::property_tree::ptree &node);

}


#endif /* OSP_CALIB_PROJECTION_PINHOLE_H */

