#ifndef OSP_CALIB_LUT_CACHE_H
#define OSP_CALIB_LUT_CACHE_H

#include <map>
#include <osp/calib/camera.h>

namespace osp {

class OSP_CALIB_EXPORT LutCache
{
public:
    using Value = osp::Matrix<osp::Point2d<float>>;

    struct Key
    {
        osp::Camera sourceModel;
        osp::Camera targetModel;
    };

    LutCache() = default;
    ~LutCache() = default;

    bool has(const Key &key);
    void put(const Key &key, const Value &lut);
    Value get(const Key &key);
    void clear();

private:
    std::vector<std::pair<Key, Value>>::iterator find(const Key &key);
    std::vector<std::pair<Key, Value>> m_storage;
    std::mutex m_mutex;
};

OSP_CALIB_EXPORT bool operator==(const LutCache::Key &lhs, const LutCache::Key &rhs);

}

#endif
