#ifndef OSP_THREAD_H
#define OSP_THREAD_H

#include <mutex>
#include <thread>

namespace osp {

class Thread
{
public:
    Thread() = default;
    virtual ~Thread() = default;

    bool start()
    {
        std::lock_guard<std::mutex> lock(m_threadMutex);
        if (m_run)
            return true;

        if (!this->threadInit())
            return false;

        m_run = true;
        m_thread = std::thread(&Thread::threadFunc, this);
        return true;
    }

    void stop()
    {
        std::lock_guard<std::mutex> lock(m_threadMutex);
        m_run = false;
        if (m_thread.joinable())
            m_thread.join();
    }

    bool running()
    {
        std::lock_guard<std::mutex> lock(m_threadMutex);
        return m_run;
    }


protected:
    virtual bool threadInit() { return true; }
    virtual void threadFunc() = 0;

private:
    bool m_run;
    std::thread m_thread;
    std::mutex m_threadMutex;
};

} // namespace osp

#endif // OSP_THREAD_H
