#ifndef ELEVATIONMAP_H
#define ELEVATIONMAP_H

#include <osp/exports/cv_export.h>
#include <osp/types/mat.h>

namespace osp
{

class OSP_CV_EXPORT ElevationMap
{
public:
    struct Config
    {        
        Config()
            : minX(-3000)
            , minY(-3000)
            , minZ(0)
            , maxX(3000)
            , maxY(3000)
            , maxZ(2000)
            , stepX(10)
            , stepY(10)
        {}

        float minX;
        float minY;
        float minZ;
        float maxX;
        float maxY;
        float maxZ;
        float stepX;
        float stepY;
    };

    ElevationMap(Config config);
    ~ElevationMap();

    Matrix<float, 1> process(Matrix<float, 3> pointCloud);

    Vec2<int> mapWorldToImage(Vec2<float> ptWorld);
    Vec2<float> mapImageToWorld(Vec2<int> ptPix);

    float minX();
    float minY();
    float minZ();
    float maxX();
    float maxY();
    float maxZ();
    float stepX();
    float stepY();

    void setMinX(float minX);
    void setMinY(float minY);
    void setMinZ(float minZ);
    void setMaxX(float maxX);
    void setMaxY(float maxY);
    void setMaxZ(float maxZ);
    void setStepX(float stepX);
    void setStepY(float stepY);

private:
    Config m_config;
};

}


#endif
