#ifndef OSP_GRPC_SERVER_H
#define OSP_GRPC_SERVER_H

#include <unordered_set>
#include <thread>
#include <grpc++/security/server_credentials.h>
#include <grpc++/server.h>
#include <osp/exports/grpc_server_export.h>

namespace osp {

namespace grpc {

class OSP_GRPC_SERVER_EXPORT Server
{
public:
    static std::unique_ptr<Server> create();

    Server();
    ~Server();

    void registerService(::grpc::Service *service);

    bool run(std::string serverIp = "0.0.0.0", uint32_t port = 50051, uint32_t portSecure = 50551);
    void shutdown();

private:
    void generateServerCredentials();
    std::string readFile(const std::string& filename);
    std::shared_ptr<::grpc::ServerCredentials> readServerCredentials();

    std::unordered_set<::grpc::Service *> m_services;
    std::unique_ptr<::grpc::Server> m_server;
    std::thread m_serverThread;
};

}

}

#endif /* OSP_GRPC_SERVER_H */
