#ifndef DEVICEMANAGER_H
#define DEVICEMANAGER_H

#include <QFuture>
#include <QFutureWatcher>
#include <QObject>
#include <osp/sdk/device.h>

class DeviceInfo : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString type READ type NOTIFY typeChanged)
    Q_PROPERTY(QString uri READ uri NOTIFY uriChanged)
    Q_PROPERTY(QString serialNumber READ serialNumber NOTIFY serialNumberChanged)
public:
    DeviceInfo(QString type, QString uri, QString serialNumber, QObject *parent = nullptr);
    virtual ~DeviceInfo();

    QString type();
    QString uri();
    QString serialNumber();

Q_SIGNALS:
    void typeChanged();
    void uriChanged();
    void serialNumberChanged();

private:
    QString m_type;
    QString m_uri;
    QString m_serialNumber;
};

class DeviceManager : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QList<QObject *> devices READ devices NOTIFY devicesChanged)
    Q_PROPERTY(bool scanning READ scanning NOTIFY scanningChanged)
public:
    DeviceManager(QObject *parent = nullptr);
    virtual ~DeviceManager();

    QList<QObject *> devices();
    bool scanning();

    Q_INVOKABLE void startRescan();

Q_SIGNALS:
    void devicesChanged();
    void scanningChanged();

private Q_SLOTS:
    void onRescanFinished();

private:
    QList<QObject *> m_devices;
    QFuture<void> m_future;
    QFutureWatcher<void> m_futureWatcher;

};


#endif // DEVICE_H
