#ifndef OSP_VIEWER_EXTRINSIC_CALIBRATOR_H
#define OSP_VIEWER_EXTRINSIC_CALIBRATOR_H

#include <QObject>
#include <QPoint>
#include <QVector>
#include <osp/qml/device.h>

class ExtrinsicCalibrator : public QObject
{
    Q_OBJECT
    Q_PROPERTY(Device *device READ device WRITE setDevice NOTIFY deviceChanged)
    Q_PROPERTY(int pointCount READ pointCount NOTIFY pointCountChanged)
    Q_PROPERTY(bool polygonReady READ polygonReady NOTIFY polygonReadyChanged)
    Q_PROPERTY(QPointF origin READ origin WRITE setOrigin NOTIFY originChanged)
    Q_PROPERTY(QPointF ptOnXAxis READ ptOnXAxis WRITE setPtOnXAxis NOTIFY ptOnXAxisChanged)
public:
    ExtrinsicCalibrator(QObject *parent = nullptr);
    virtual ~ExtrinsicCalibrator();

    void setDevice(Device *device);
    Device *device();

    void setOrigin(QPointF origin);
    QPointF origin();

    void setPtOnXAxis(QPointF ptOnXAxis);
    QPointF ptOnXAxis();

    Q_INVOKABLE void addPoint(QPointF pt);
    Q_INVOKABLE void removePoint(int idx);
    Q_INVOKABLE QPointF point(int idx);
    int pointCount();
    bool polygonReady();

    Q_INVOKABLE bool calibrate();
    Q_INVOKABLE void reset();

signals:
    void deviceChanged();
    void originChanged();
    void ptOnXAxisChanged();
    void pointCountChanged();
    void polygonReadyChanged();

private slots:
    void onFrameChanged();

private:
    Device *m_device;
    QPointF m_origin;
    QPointF m_ptOnXAxis;
    QVector<QPointF> m_points;
    bool m_polygonReady;

    std::shared_ptr<osp::Frame> m_latestFrame;
    std::mutex m_frameMutex;
};

#endif
