#ifndef FRAME_H
#define FRAME_H

#include <QObject>
#include <osp/sdk/device.h>
#include "matrix.h"

class FrameMetadata : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString timestamp READ timestamp NOTIFY timestampChanged)
    Q_PROPERTY(float analogGain READ analogGain NOTIFY analogGainChanged)
    Q_PROPERTY(float exposureTime READ exposureTime NOTIFY exposureTimeChanged)
    Q_PROPERTY(unsigned int sensitivity READ sensitivity NOTIFY sensitivityChanged)
    Q_PROPERTY(float digitalGain READ digitalGain NOTIFY digitalGainChanged)
    Q_PROPERTY(float sceneLux READ sceneLux NOTIFY sceneLuxChanged)
public:
    FrameMetadata(QObject *parent = nullptr);
    virtual ~FrameMetadata();

    QString timestamp() const;
    float analogGain() const;
    float exposureTime() const;
    unsigned int sensitivity() const;
    float digitalGain() const;
    float sceneLux() const;

    void setTimestamp(QString timestamp);
    void setAnalogGain(float analogGain);
    void setExposureTime(float exposureTime);
    void setSensitivity(unsigned int sensitivity);
    void setDigitalGain(float digitalGain);
    void setSceneLux(float sceneLux);

signals:
    void timestampChanged();
    void analogGainChanged();
    void exposureTimeChanged();
    void sensitivityChanged();
    void digitalGainChanged();
    void sceneLuxChanged();

private:
    QString m_timestamp;
    float m_analogGain;
    float m_exposureTime;
    uint32_t m_sensitivity;
    float m_digitalGain;
    float m_sceneLux;
};

class Frame : public QObject
{
    Q_OBJECT
    Q_PROPERTY(FrameMetadata *metadata READ metadata NOTIFY metadataChanged)
    Q_PROPERTY(Matrix *sourceImage0 READ sourceImage0 NOTIFY sourceImage0Changed)
    Q_PROPERTY(Matrix *sourceImage1 READ sourceImage1 NOTIFY sourceImage1Changed)
    Q_PROPERTY(Matrix *sourceImage2 READ sourceImage2 NOTIFY sourceImage2Changed)
    Q_PROPERTY(Matrix *image READ image NOTIFY imageChanged)
    Q_PROPERTY(Matrix *distanceMap READ distanceMap NOTIFY distanceMapChanged)
    Q_PROPERTY(Matrix *pointCloud READ pointCloud NOTIFY pointCloudChanged)
public:
    Frame(const std::shared_ptr<osp::Frame> &frame, QObject *parent = nullptr);
    virtual ~Frame();

    FrameMetadata *metadata();
    Matrix *sourceImage0();
    Matrix *sourceImage1();
    Matrix *sourceImage2();
    Matrix *image();
    Matrix *distanceMap();
    Matrix *pointCloud();

    std::shared_ptr<osp::Frame> internalFrame();

signals:
    void metadataChanged();
    void sourceImage0Changed();
    void sourceImage1Changed();
    void sourceImage2Changed();
    void imageChanged();
    void distanceMapChanged();
    void pointCloudChanged();

private:
    std::shared_ptr<osp::Frame> m_frame;
    FrameMetadata *m_metadata;
    Matrix *m_sourceImage0;
    Matrix *m_sourceImage1;
    Matrix *m_sourceImage2;
    Matrix *m_image;
    Matrix *m_distanceMap;
    Matrix *m_pointCloud;
};


#endif // FRAME_H
