#ifndef IMAGEDISPLAY_H
#define IMAGEDISPLAY_H

#include <QtQuick/QQuickPaintedItem>
#include <QMutex>
#include <QImage>

#include "matrix.h"

class ImageDisplay : public QQuickPaintedItem
{
    Q_OBJECT
    Q_PROPERTY(Matrix *data READ data WRITE setData NOTIFY dataChanged)
    Q_PROPERTY(float offsetX READ offsetX NOTIFY offsetXChanged)
    Q_PROPERTY(float offsetY READ offsetY NOTIFY offsetYChanged)
    Q_PROPERTY(float scaleX READ scaleX NOTIFY scaleXChanged)
    Q_PROPERTY(float scaleY READ scaleY NOTIFY scaleYChanged)
public:
    explicit ImageDisplay(QQuickItem *parent = nullptr);
    virtual ~ImageDisplay();

    Matrix* data();
    void setData(Matrix* data);

    float offsetX();
    float offsetY();
    float scaleX();
    float scaleY();

    Q_INVOKABLE QPointF mapToImage(QPointF ptItem);
    Q_INVOKABLE QPointF mapFromImage(QPointF ptImg);

Q_SIGNALS:
    void dataChanged();
    void offsetXChanged();
    void offsetYChanged();
    void scaleXChanged();
    void scaleYChanged();

protected:
    void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry);
    void paint(QPainter* painter) override;

private Q_SLOTS:
    void updateMe();

private:
    void updateScale();
    Matrix *m_data;
    size_t m_imageWidth;
    size_t m_imageHeight;
    float m_scaleX;
    float m_scaleY;
    float m_offsetX;
    float m_offsetY;
};

#endif // IMAGEDISPLAYCOMPONENT_H
