#ifndef PLANEENTITY_H
#define PLANEENTITY_H

#include <QEntity>
#include <QNode>
#include <Qt3DExtras>
#include <Qt3DRender>

class PlaneEntity : public Qt3DCore::QEntity
{
    Q_OBJECT
    Q_PROPERTY(float minX READ minX WRITE setMinX NOTIFY minXChanged)
    Q_PROPERTY(float maxX READ maxX WRITE setMaxX NOTIFY maxXChanged)
    Q_PROPERTY(float minY READ minY WRITE setMinY NOTIFY minYChanged)
    Q_PROPERTY(float maxY READ maxY WRITE setMaxY NOTIFY maxYChanged)
    Q_PROPERTY(float stepX READ stepX WRITE setStepX NOTIFY stepXChanged)
    Q_PROPERTY(float stepY READ stepY WRITE setStepY NOTIFY stepYChanged)
public:
    PlaneEntity(Qt3DCore::QNode *parent = nullptr);
    ~PlaneEntity();

    float minX();
    float maxX();
    float minY();
    float maxY();
    float stepX();
    float stepY();

    void setMinX(float minX);
    void setMaxX(float maxX);
    void setMinY(float minY);
    void setMaxY(float maxY);
    void setStepX(float stepX);
    void setStepY(float stepY);

signals:
    float minXChanged();
    float maxXChanged();
    float minYChanged();
    float maxYChanged();
    float stepXChanged();
    float stepYChanged();

private Q_SLOTS:
    void updateGeometry();

private:
    float m_minX;
    float m_maxX;
    float m_minY;
    float m_maxY;
    float m_stepX;
    float m_stepY;

    Qt3DCore::QTransform *m_transform;
    Qt3DExtras::QPhongMaterial *m_material;
    Qt3DRender::QGeometryRenderer *m_renderer;
    Qt3DRender::QGeometry *m_geometry;
    Qt3DRender::QAttribute *m_positionAttribute;
};

#endif // PLANEENTITY_H
