#ifndef RECORDER_H
#define RECORDER_H

#include <condition_variable>
#include <thread>
#include <QUrl>
#include "device.h"

class Recorder : public QObject
{
    Q_OBJECT
    Q_PROPERTY(Device* device READ device WRITE setDevice NOTIFY deviceChanged)
    Q_PROPERTY(bool running READ running NOTIFY runningChanged)
    Q_PROPERTY(QUrl outputDirectory READ outputDirectory WRITE setOutputDirectory NOTIFY outputDirectoryChanged)
    Q_PROPERTY(int frameCount READ frameCount NOTIFY frameCountChanged)
    Q_PROPERTY(bool writePointcloud READ writePointcloud WRITE setWritePointcloud NOTIFY writePointcloudChanged)
public:
    Recorder(QObject *parent = nullptr);
    virtual ~Recorder();

    Device *device();
    void setDevice(Device *device);

    bool running();

    QUrl outputDirectory();
    void setOutputDirectory(QUrl dir);

    int frameCount();

    bool writePointcloud() const;
    void setWritePointcloud(bool writePointcloud);

    Q_INVOKABLE void start();
    Q_INVOKABLE void stop();

Q_SIGNALS:
    void deviceChanged();
    void runningChanged();
    void outputDirectoryChanged();
    void frameCountChanged();
    void writePointcloudChanged();

private:
    void onDeviceReadyChanged();
    void onFrameChanged();
    void setRunning(bool running);
    void setFrameCount(int frameCount);
    void connectFrameSignal();
    void disconnectFrameSignal();
    void updateCurrentOutputDirectory();
    void createDirIfNotExist(std::string path);
    void writeFrame(const std::shared_ptr<osp::Frame> &frame);
    void saveCameraParameters();
    void writeLoop();

    Device *m_device;
    bool m_running;
    bool m_writePointcloud;
    QUrl m_outputDirectory;
    int m_frameCount;
    std::string m_currentOutputDirectory;

    std::mutex m_frameMutex;
    std::condition_variable m_frameAvailable;
    std::shared_ptr<osp::Frame> m_frameBuffer;
    std::thread m_writerThread;
};


#endif // DEVICE_H
