#ifndef OSP_TYPES_BUFFER_H
#define OSP_TYPES_BUFFER_H

#include <memory>

namespace osp {

template<typename T>
class Buffer
{
public:
    Buffer();
    explicit Buffer(size_t size);
    Buffer(size_t size, T *data);
    virtual ~Buffer();
    Buffer(const Buffer<T> & other);
    Buffer(Buffer<T> && other);

    Buffer<T> & operator =(Buffer<T> other);
    T & operator [](const size_t idx);
    bool empty() const;
    void swap(Buffer<T> & other) noexcept;

    T *data() const;
    size_t size() const;

    void allocate(size_t size);
    void release();

    Buffer<T> clone() const;

private:
    size_t m_size;
    std::shared_ptr<T> m_data;
    T* m_dataPtr;
};

}

#include <osp/types/details/buffer.h>

#endif
