#ifndef OSP_TYPES_BUFFER_DETAILS_H
#define OSP_TYPES_BUFFER_DETAILS_H

namespace osp {

template<typename T>
Buffer<T>::Buffer()
    : m_size(0)
    , m_data(nullptr)
    , m_dataPtr(nullptr)
{
}

template<typename T>
Buffer<T>::Buffer(size_t size)
    : m_data(nullptr)
    , m_dataPtr(nullptr)
{
    allocate(size);
}

template<typename T>
Buffer<T>::Buffer(size_t size, T *data)
    : m_size(size)
    , m_dataPtr(data)
{

}

template<typename T>
Buffer<T>::~Buffer()
{
    release();
}

template<typename T>
Buffer<T>::Buffer(const Buffer<T> & other)
    : m_size(other.m_size)
    , m_data(other.m_data)
    , m_dataPtr(other.m_dataPtr)
{
}

template<typename T>
Buffer<T>::Buffer(Buffer<T> && other)
    : m_size(std::move(other.m_size))
    , m_data(std::move(other.m_data))
    , m_dataPtr(std::move(other.m_dataPtr))
{
}

template<typename T>
Buffer<T> & Buffer<T>::operator =(Buffer<T> other)
{
    other.swap(*this);
    return *this;
}

template<typename T>
T & Buffer<T>::operator [](const size_t idx)
{
    assert(idx < m_size);
    return m_dataPtr[idx];
}

template<typename T>
bool Buffer<T>::empty() const
{
    return m_size == 0;
}

template<typename T>
void Buffer<T>::swap(Buffer<T> & other) noexcept
{
    std::swap(this->m_size, other.m_size);
    std::swap(this->m_data, other.m_data);
    std::swap(this->m_dataPtr, other.m_dataPtr);
}

template<typename T>
T *Buffer<T>::data() const
{
    return m_dataPtr;
}

template<typename T>
size_t Buffer<T>::size() const
{
    return m_size;
}

template<typename T>
void Buffer<T>::allocate(size_t size)
{
    m_size = std::move(size);
    m_data = std::shared_ptr<T>(new T[m_size], std::default_delete<T[]>());
    m_dataPtr = m_data.get();
}

template<typename T>
void Buffer<T>::release()
{
    m_size = 0;
    m_data.reset();
    m_dataPtr = nullptr;
}

template<typename T>
Buffer<T> Buffer<T>::clone() const
{
    Buffer<T> buf(m_size);
    std::copy(data(), data() + m_size, buf.data());
    return std::move(buf);
}

}

#endif
