#ifndef OSP_MEMORY_H
#define OSP_MEMORY_H

#include <algorithm>
#include <stdexcept>

namespace osp
{

struct HostMemory {};

template <typename T, typename MemoryTag>
class DefaultAllocator
{
};

template <typename T>
class DefaultAllocator<T, HostMemory>
{
public:
    using ValueType = T;
    using PtrType = ValueType *;

    PtrType allocate(std::size_t size);
    void deallocate(PtrType data, std::size_t /*size*/);
};

template <typename T>
typename DefaultAllocator<T, HostMemory>::PtrType DefaultAllocator<T, HostMemory>::allocate(std::size_t size)
{
    return new ValueType[size];
}

template <typename T>
void DefaultAllocator<T, HostMemory>::deallocate(PtrType data, std::size_t /*size*/)
{
    delete[] data;
}

template <typename T, typename MemoryTag = HostMemory>
class Memory
{
public:
    using ValueType = T;
    using PtrType = ValueType *;

    Memory(std::size_t size);
    Memory(PtrType data, std::size_t size,  bool takeOwnership = false);
    ~Memory();

    PtrType data();
    PtrType data() const;
    std::size_t size() const;
    void allocate(std::size_t size);
    void deallocate();

private:
    DefaultAllocator<ValueType, MemoryTag> m_allocator;
    PtrType m_data;
    bool m_isOwner;
    std::size_t m_size;
};



template <typename T, typename MemoryTag>
Memory<T, MemoryTag>::Memory(std::size_t size)
{
    allocate(size);
}

template <typename T, typename MemoryTag>
Memory<T, MemoryTag>::Memory(PtrType data, size_t size, bool takeOwnership)
{
    m_data = data;
    m_size = size;
    m_isOwner = takeOwnership;
}

template <typename T, typename MemoryTag>
Memory<T, MemoryTag>::~Memory()
{
    deallocate();
}

template <typename T, typename MemoryTag>
typename Memory<T, MemoryTag>::PtrType Memory<T, MemoryTag>::data()
{
    return m_data;
}

template <typename T, typename MemoryTag>
typename Memory<T, MemoryTag>::PtrType Memory<T, MemoryTag>::data() const
{
    return m_data;
}

template <typename T, typename MemoryTag>
std::size_t Memory<T, MemoryTag>::size() const
{
    return m_size;
}

template <typename T, typename MemoryTag>
void Memory<T, MemoryTag>::allocate(std::size_t size)
{
    if (m_size == size && m_isOwner)
        return;

    deallocate();
    m_data = m_allocator.allocate(size);
    m_isOwner = true;
    m_size = size;
}

template <typename T, typename MemoryTag>
void Memory<T, MemoryTag>::deallocate()
{
    if (!m_data || !m_isOwner)
        return;

    m_allocator.deallocate(m_data, m_size);
    m_data = nullptr;
    m_size = 0;
}

template <typename T, typename MemoryTagSrc, typename MemoryTagDst>
void copyMemory(const Memory<T, MemoryTagSrc> &/*src*/, Memory<T, MemoryTagDst> &/*dst*/)
{
    static_assert(sizeof(T) == 0, "Copy function not implemented for given memory types.");
}

template <typename T>
void copyMemory(const Memory<T, HostMemory> &src, Memory<T, HostMemory> &dst)
{
    dst.allocate(src.size());
    std::copy(src.data(), src.data() + src.size(), dst.data());
}

}

#endif

