#ifndef TS_QUATERNION_H
#define TS_QUATERNION_H

#include <osp/types/mat.h>
#include <osp/exports/types_export.h>

namespace osp {

class OSP_TYPES_EXPORT Quaternion
{
public:
    Quaternion();
    Quaternion(double x, double y, double z, double w);
    Quaternion(Mat<double, 3, 3> rotationMat);
    Quaternion(Vec<double, 3> rodrigues);

    void setX(double x);
    void setY(double y);
    void setZ(double z);
    void setW(double w);

    double x() const;
    double y() const;
    double z() const;
    double w() const;

    Quaternion normalized() const;
    Quaternion inverse() const;

    Mat<double, 3, 3> toRotMat();

private:
    void normalize();
    double m_x;
    double m_y;
    double m_z;
    double m_w;
};

inline std::ostream &operator <<(std::ostream &stream, const Quaternion &q)
{
    stream << "[ ";
    stream << q.x() << ", ";
    stream << q.y() << ", ";
    stream << q.z() << ", ";
    stream << q.w();
    stream << " ]" << std::endl;

    return stream;
}

}

#endif // TS_QUATERNION_H
