#include <osp/cv/vptz.h>
#include <osp/io/imagefile.h>
#include <osp/types/mathelper.h>
#include <opencv2/highgui.hpp>

using namespace osp;

int main()
{
    // configure virtual pan-tilt-zoom camera
    VPtz vptz;
    vptz.setSourceCalib(Camera::readFromFile("calib.xml"));
    vptz.setFocalLength(200);
    vptz.setImageResolution(640, 480);
    vptz.setOrientation(0, 0, 0);

    // setup windows
    cv::namedWindow("Source", cv::WINDOW_NORMAL);
    cv::namedWindow("VPTZ", cv::WINDOW_NORMAL);

    // read sample image
    Matrix<uint8_t, 3> imageSource = io::readImageFromFile("sample.png");
    cv::imshow("Source", matrixToCvMat(imageSource));

    int key;
    do
    {
        // create perspective view
        Matrix<uint8_t, 3> imageTarget = vptz.transform(imageSource);
        cv::imshow("VPTZ", matrixToCvMat(imageTarget));
        
        // user controls
        key = cv::waitKey(10);
        switch (key)
        {
        case 'w':
            vptz.setTilt(vptz.tilt() + 0.1);
            break;
        case 's':
            vptz.setTilt(vptz.tilt() - 0.1);
            break;
        case 'd':
            vptz.setPan(vptz.pan() + 0.1);
            break;
        case 'a':
            vptz.setPan(vptz.pan() - 0.1);
            break;
        case 'e':
            vptz.setRoll(vptz.roll() + 0.1);
            break;
        case 'q':
            vptz.setRoll(vptz.roll() - 0.1);
            break;
        case '+':
            vptz.setFocalLength(vptz.focalLength() + 20);
            break;
        case '-':
            vptz.setFocalLength(vptz.focalLength() - 20);
            break;
        }
    }
    while (key != 'x');

    return EXIT_SUCCESS;
}
