#!/bin/bash

function generate_csr()
{
  # Generate valid Server Key/Cert
  openssl genrsa -passout pass:1234 -des3 -out hemistereo.key 4096
  openssl req -passin pass:1234 -new -key hemistereo.key -out hemistereo.csr -subj  "/C=DE/ST=Sachsen/L=Chemnitz/O=3dvisionlabs GmbH/OU=HemiStereo/CN=hemistereo"
  # Remove passphrase from the Server Key
  openssl rsa -passin pass:1234 -in hemistereo.key -out hemistereo.key
}

function generate_root_key_password()
{
  echo "Please set a password with at least 4 characters to secure your root key."
  while true; do
    read -sp "Password: " ca_pwd
    echo
    read -sp "Confirm: " ca_pwd_confirm
    echo

    if [ "$ca_pwd" == "$ca_pwd_confirm" ]; then
      if (( ${#ca_pwd} >= 4 )); then
        break
      else
        echo "Password must have at least 4 characters!"
      fi
    else
      echo "Passwords do not match!"
    fi
  done
}

function find_root_key()
{
  read -p "Path to the root certificate: [ca.crt] " ca_crt
  read -p "Path to the private key: [ca.key] " ca_key
  if [ -z "$ca_crt" ]; then
    ca_crt="ca.crt"
  fi
  if [ -z "$ca_key" ]; then
    ca_key="ca.key"
  fi
}

function generate_root_key()
{
  generate_root_key_password
  openssl genrsa -passout pass:$ca_pwd -des3 -out ca.key 4096
  openssl req -passin pass:$ca_pwd -new -x509 -days 7300 -key ca.key -out ca.crt -subj  "/C=DE/ST=Sachsen/L=Chemnitz/O=3dvisionlabs GmbH/OU=HemiStereo/CN=Root CA"

  ca_crt="ca.crt"
  ca_key="ca.key"
}

function self_sign()
{
  echo
  echo "You need a root key for signing the server certificate. If you have"
  echo "multiple HemiStereo sensors, it is recommended to sign the server"
  echo "certificates using the same root key. This eliminates the need to"
  echo "copy a separate certificate for each sensor to the clients."
  echo
  read -p "Do you want to generate a new one? [Y/n] " yn

  ca_key=""
  ca_pwd=""
  case $yn in
    [Nn]* ) find_root_key;;
    * ) generate_root_key
  esac

  if [ -n "$ca_pwd" ]; then
    openssl x509 -req -passin pass:$ca_pwd -days 7300 -in hemistereo.csr -CA $ca_crt -CAkey $ca_key -set_serial 01 -out hemistereo.crt
  else
    openssl x509 -req -days 7300 -in hemistereo.csr -CA $ca_crt -CAkey $ca_key -set_serial 01 -out hemistereo.crt
  fi
}

pwd=`pwd`

echo "HemiStereo Server Certificate Generator"
echo "======================================="
echo

if [ -f hemistereo.key ] || [ -f hemistereo.crt ]; then
  echo "Server certificate and/or private key (hemistereo.crt/hemistereo.key)"
  read -p "already exist in the current directory. Overwrite files? [y/N] " yn
  case $yn in
      [Yy]* ) :;;
      * ) exit;;
  esac
fi

generate_csr

echo
echo "The server certificate has to be signed by 3dvisionlabs. There are two"
echo "ways of getting a signed certificate. The preferred way is to receive"
echo "a signed certificate from 3dvisionlabs."
echo
echo "Alternatively, you can also sign the server certificate by yourself."
echo "The downside of this solution is that you have to copy the self-generated"
echo "root certificate to each client that should be able to connect to the"
echo "HemiStereo device."
echo

is_self_signed=0
ca_crt=""

read -p "Do you want to self sign the generated server certificate? [y/N] " yn
case $yn in
  [Yy]* )
    self_sign
    is_self_signed=1;;
  * ) :;;
esac

echo

if [ $is_self_signed -eq 1 ]; then
  echo "To use the server certificate, copy hemistereo.key and hemistereo.crt to"
  echo "/etc/3dvisionlabs/osp/certs on your HemiStereo sensor."

  echo "Also copy your root certificate ($ca_crt) to /etc/3dvisionlabs/osp/certs"
  echo "(Linux) or C:\ProgramData\3dvisionlabs\osp\certs (Windows) on the clients"
  echo "that need to esablish a TLS connection to the HemiStereo sensor."
else
  echo "Please, send the generated certificate signing request (hemistereo.csr)"
  echo "to support@3dvisionlabs.com. 3dvisionlabs will sign the certificate and"
  echo "and send it back to you."
  echo
  echo "After receiving the signed certificate, you have to copy the certificate"
  echo "(hemistereo.crt) and the key (hemistereo.key) to /etc/3dvisionlabs/osp/certs"
  echo "on your HemiStereo sensor."
fi
